<?php

/**
 * @file
 * Administrative forms for the Forte.Net module.
 */

/**
 * Form callback: allows the user to capture a prior authorization.
 */
function commerce_forte_aim_capture_form($form, &$form_state, $order, $transaction) {
  $form_state['order'] = $order;
  $form_state['transaction'] = $transaction;

  // Load and store the payment method instance for this transaction.
  $payment_method = commerce_payment_method_instance_load($transaction->instance_id);
  $form_state['payment_method'] = $payment_method;

  $balance = commerce_payment_order_balance($order);

  if ($balance['amount'] > 0 && $balance['amount'] < $transaction->amount) {
    $default_amount = $balance['amount'];
  }
  else {
    $default_amount = $transaction->amount;
  }

  // Convert the price amount to a user friendly decimal value.
  $default_amount = number_format(commerce_currency_amount_to_decimal($default_amount, $transaction->currency_code), 2, '.', '');

  $description = implode('<br />', array(
    t('Authorization: @amount', array('@amount' => commerce_currency_format($transaction->amount, $transaction->currency_code))),
    t('Order balance: @balance', array('@balance' => commerce_currency_format($balance['amount'], $balance['currency_code']))),
  ));

  $form['amount'] = array(
    '#type' => 'textfield',
    '#title' => t('Capture amount'),
    '#description' => $description,
    '#default_value' => $default_amount,
    '#field_suffix' => check_plain($transaction->currency_code),
    '#size' => 16,
    '#attributes' => array(
      'readonly' => 'readonly',
      'style' => 'background: none repeat scroll 0 0 #EAEAEA;',
    ),
  );

  $form = confirm_form($form,
    t('What amount do you want to capture?'),
    'admin/commerce/orders/' . $order->order_id . '/payment',
    '',
    t('Capture'),
    t('Cancel'),
    'confirm'
  );

  return $form;
}

/**
 * Validate handler: ensure a valid amount is given.
 */
function commerce_forte_aim_capture_form_validate($form, &$form_state) {
  $transaction = $form_state['transaction'];
  $amount = $form_state['values']['amount'];

  // Ensure a positive numeric amount has been entered for capture.
  if (!is_numeric($amount) || $amount <= 0) {
    form_set_error('amount', t('You must specify a positive numeric amount to capture.'));
  }

  // Ensure the amount is less than or equal to the authorization amount.
  if ($amount > commerce_currency_amount_to_decimal($transaction->amount, $transaction->currency_code)) {
    form_set_error('amount', t('You cannot capture more than you authorized through Forte.Net.'));
  }

  // If the authorization has expired, display an error message and redirect.
  if (time() - $transaction->created > 86400 * 30) {
    drupal_set_message(t('This authorization has passed its 30 day limit cannot be captured.'), 'error');
    drupal_goto('admin/commerce/orders/' . $form_state['order']->order_id . '/payment');
  }
}

/**
 * Submit handler: process a prior authorization capture via AIM.
 */
function commerce_forte_aim_capture_form_submit($form, &$form_state) {
  $transaction = $form_state['transaction'];
  $order = $form_state['order'];
  $amount = number_format($form_state['values']['amount'], 2, '.', '');

  if ($transaction->revision_timestamp == $transaction->created) {
    $capture_data = $transaction->payload[$transaction->revision_timestamp];
  }
  else {
    $capture_data = $transaction->payload[$transaction->created];
  }
  $order_wrapper = entity_metadata_wrapper('commerce_order', $order);
  // Prepare the billing address for use in the request.
  if (isset($order->commerce_customer_billing) && $order_wrapper->commerce_customer_billing->value()) {
    $billing_address = $order_wrapper->commerce_customer_billing->commerce_customer_address->value();

    if (empty($billing_address['first_name'])) {
      $name_parts = explode(' ', $billing_address['name_line']);
      $billing_address['first_name'] = array_shift($name_parts);
      $billing_address['last_name'] = implode(' ', $name_parts);
    }

  }
  $capture_full_name = substr($billing_address['first_name'], 0, 50) . ' ' . substr($billing_address['last_name'], 0, 50);

  // Build a name-value pair array for this transaction.
  $nvp = array(
    'account_id' => 'act_' . $form_state['payment_method']['settings']['restacct_id'],
    'action' => 'Capture',
    'authorization_code' => $capture_data['authorization_code'],
    'entered_by' => $capture_full_name,
  );

  $merchant_ids = 'loc_' . $form_state['payment_method']['settings']['mercacct_id'];
  $restacct_id = 'act_' . $form_state['payment_method']['settings']['restacct_id'];
  $act_loc_url = 'accounts/' . $restacct_id . '/locations/' . $merchant_ids . '/transactions/';
  $service_url = commerce_forte_aim_server_url($form_state['payment_method']['settings']['txn_mode']) . $act_loc_url . $capture_data['transaction_id'];

  $apiloginid = $form_state['payment_method']['settings']['login'];
  $securetransactionkey = $form_state['payment_method']['settings']['tran_key'];
  $rest_hash = $apiloginid . ':' . $securetransactionkey;
  $auth_token = base64_encode($rest_hash);
  $curlopt_customrequests = "PUT";

  // Submit the request to Forte.Net.
  $response = commerce_forte_curl_request($service_url, $auth_token, $curlopt_customrequests, $form_state['payment_method'], $nvp);

  // Update and save the transaction based on the response.
  $transaction->payload[REQUEST_TIME] = $response;

  // If we didn't get an approval response code...
  if ($response['response']['response_desc'] != 'APPROVED') {
    // Display an error message but leave the transaction pending.
    drupal_set_message(t('This order already captured.'), 'error');
    drupal_set_message(check_plain($response), 'error');
  }
  else {
    drupal_set_message(t('Transaction successfully captured.'));

    // Update the transaction amount to the actual capture amount.
    $transaction->amount = commerce_currency_decimal_to_amount($amount, $transaction->currency_code);

    // Set the remote and local status accordingly.
    $transaction->status = COMMERCE_PAYMENT_STATUS_SUCCESS;

    // Append a capture indication to the result message.
    $transaction->message .= '<br />' . t('<b>Captured: @date</b>', array('@date' => format_date(REQUEST_TIME, 'short')));
  }
  // Automatically status changes while capture.
  $target_step_status = 'completed';
  commerce_order_status_update($order, $target_step_status);
  commerce_payment_transaction_save($transaction);

  $form_state['redirect'] = 'admin/commerce/orders/' . $form_state['order']->order_id . '/payment';
}

/**
 * Form callback: allows the user to void a transaction.
 */
function commerce_forte_aim_void_form($form, &$form_state, $order, $transaction) {
  $form_state['order'] = $order;
  $form_state['transaction'] = $transaction;

  // Load and store the payment method instance for this transaction.
  $payment_method = commerce_payment_method_instance_load($transaction->instance_id);
  $form_state['payment_method'] = $payment_method;

  $form['markup'] = array(
    '#markup' => t('Are you sure that you want to void this transaction?'),
  );

  $form = confirm_form($form,
    t('Are you sure that you want to void this transaction?'),
    'admin/commerce/orders/' . $order->order_id . '/payment',
    '',
    t('Void'),
    t('Cancel'),
    'confirm'
  );

  return $form;
}

/**
 * Submit handler: process the void request.
 */
function commerce_forte_aim_void_form_submit($form, &$form_state) {
  $transaction = $form_state['transaction'];
  $order = $form_state['order'];

  if ($transaction->revision_timestamp == $transaction->created) {
    $capture_data = $transaction->payload[$transaction->revision_timestamp];
  }
  else {
    $capture_data = $transaction->payload[$transaction->created];
  }
  $order_wrapper = entity_metadata_wrapper('commerce_order', $order);
  // Prepare the billing address for use in the request.
  if (isset($order->commerce_customer_billing) && $order_wrapper->commerce_customer_billing->value()) {
    $billing_address = $order_wrapper->commerce_customer_billing->commerce_customer_address->value();

    if (empty($billing_address['first_name'])) {
      $name_parts = explode(' ', $billing_address['name_line']);
      $billing_address['first_name'] = array_shift($name_parts);
      $billing_address['last_name'] = implode(' ', $name_parts);
    }

  }
  $capture_full_name = substr($billing_address['first_name'], 0, 50) . ' ' . substr($billing_address['last_name'], 0, 50);

  // Build a name-value pair array for this transaction.
  $nvp = array(
    'account_id' => 'act_' . $form_state['payment_method']['settings']['restacct_id'],
    'action' => 'void',
    'authorization_code' => $capture_data['authorization_code'],
    'entered_by' => $capture_full_name,
  );

  $merchant_ids = 'loc_' . $form_state['payment_method']['settings']['mercacct_id'];
  $restacct_id = 'act_' . $form_state['payment_method']['settings']['restacct_id'];
  $act_loc_url = 'accounts/' . $restacct_id . '/locations/' . $merchant_ids . '/transactions/';
  $service_url = commerce_forte_aim_server_url($form_state['payment_method']['settings']['txn_mode']) . $act_loc_url . $capture_data['transaction_id'];

  $apiloginid = $form_state['payment_method']['settings']['login'];
  $securetransactionkey = $form_state['payment_method']['settings']['tran_key'];
  $rest_hash = $apiloginid . ':' . $securetransactionkey;
  $auth_token = base64_encode($rest_hash);
  $curlopt_customrequests = "PUT";

  // Submit the request to Forte.Net.
  $response = commerce_forte_curl_request($service_url, $auth_token, $curlopt_customrequests, $form_state['payment_method'], $nvp);

  // Update and save the transaction based on the response.
  $transaction->payload[REQUEST_TIME] = $response;

  // If we got an approval response code...
  if ($response['response']['response_desc'] == 'APPROVED') {
    drupal_set_message(t('Transaction successfully voided.'));

    // Set the remote and local status accordingly.
    $transaction->status = COMMERCE_PAYMENT_STATUS_FAILURE;

    // Update the transaction message to show that it has been voided.
    $transaction->message .= '<br />' . t('<b>Voided: @date</b>', array('@date' => format_date(REQUEST_TIME, 'short')));

    // Zero out transaction amount before saving,
    // so original amount is not saved as a positive revenue transaction.
    // Forte.Net settles these as "0.00" (see the payload on a void response).
    $transaction->amount = '000';
  }
  else {
    drupal_set_message(t('Void failed: @reason', array('@reason' => $response['response']['response_desc'])), 'error');
  }
  // Automatically status changes while void.
  $target_step_status = 'canceled';
  commerce_order_status_update($order, $target_step_status);
  commerce_payment_transaction_save($transaction);

  $form_state['redirect'] = 'admin/commerce/orders/' . $form_state['order']->order_id . '/payment';
}
